use crate::models::spotify;
use crate::services::spotify as Spotify;
use crate::{middlewares::error::ErrorResponse, models::tracks::Tracks};
use actix_web::{get, web, HttpResponse, Result, Scope};
use serde::{Deserialize, Serialize};

pub fn routes() -> Scope {
    web::scope("/search").service(search)
}

#[derive(Deserialize)]
struct SearchQuery {
    q: String,
}

#[derive(Serialize)]
struct SearchResponse {
    tracks: Vec<SearchTracks>,
}

#[derive(Serialize)]
struct SearchTracks {
    pub id: String,
}

impl From<spotify::Track> for SearchTracks {
    fn from(value: spotify::Track) -> SearchTracks {
        SearchTracks { id: value.id }
    }
}

#[get("")]
async fn search(query: web::Query<SearchQuery>) -> Result<HttpResponse, ErrorResponse> {
    let spotify = Spotify::instance().await;
    let search = spotify.search(&query.q).await.unwrap();

    Ok(HttpResponse::Ok().json(SearchResponse {
        tracks: search
            .tracks
            .items
            .into_iter()
            .map(SearchTracks::from)
            .collect::<Vec<SearchTracks>>(),
    }))
}
